// tbromtest - switching ROM banks manually, 'Lazycow 2020

#include <stdio.h>

typedef unsigned char  ubyte;
typedef unsigned short uword;

#define BANKSELECT ((ubyte*)0x8000)
#define BANKDATA   ((ubyte*)0x8001)
#define MIRRORING  ((ubyte*)0xA000)

 // Q&D MMC3 bank select
//
void Switch(ubyte bank) {
  *BANKSELECT=2+4;    *BANKDATA=bank<<1;
  *BANKSELECT=1+2+4;  *BANKDATA=(bank<<1)+1;
}

 //
//
void Write(void* p) {
	printf("%s",p);
}

  //
 // rom bank #0, bank placement by __section
//
__section("rodata0") const ubyte b0DataTab[]={0};
__section("rodata0") const char* const b0TextTab[]={"hello bank #0"};
__section("text0") void Bank0Function() {
	static const char insideText[]="insideText0\n";
	Write(insideText);
	Write("inside of Bank0Function()\n");
}

  //
 // rom bank #1, bank placement by __bank
//
__bank(1) const ubyte b1DataTab[]={1};
__bank(1) const char* const b1TextTab[]={"hello bank #1"};
__bank(1) void Bank1Function() {
	static const char insideText[]="insideText1\n";
	Write(insideText);
	Write("inside of Bank1Function()\n");
}


  //
 // rom bank #2, bank placement by pragma
//
#pragma bank 2
const ubyte b2DataTab[]={2};
const char* const b2TextTab[]={"hello bank #2"};
void Bank2Function() {
	static const char insideText[]="insideText2\n";
	Write(insideText);
	Write("inside of Bank2Function()\n");
}
#pragma bank -1


  //
 // common rom bank (default)
//
const ubyte commonDataTab[]={9};
const char* const commonTextTab[]={"hello common bank"};

 //
//
char initTest[]="init ok\n";

int main() {
	char helpBuffer[2];  helpBuffer[1]='\0';

 *MIRRORING=0; // vbcc's stdout currently only support vertical mirroring

	Write("Hello TBROM bankswitching test!\n\n");

	Write(initTest);

	// show text and integer from common bank
	helpBuffer[0]='0'+commonDataTab[0];
	Write(commonTextTab[0]);  Write("\n");
	Write(helpBuffer);  Write("\n\n");

	// show text and integer from bank #0, call function from bank #0
	Switch(0);
	helpBuffer[0]='0'+b0DataTab[0];
	Write(b0TextTab[0]);  Write("\n");
	Write(helpBuffer);  Write("\n");
	Bank0Function();  Write("\n");

	// show text and integer from bank #1, call function from bank #1
	Switch(1);
	helpBuffer[0]='0'+b1DataTab[0];
	Write(b1TextTab[0]);  Write("\n");
	Write(helpBuffer);  Write("\n");
	Bank1Function();  Write("\n");

	// show text and integer from bank #2, call function from bank #2
	Switch(2);
	helpBuffer[0]='0'+b2DataTab[0];
	Write(b2TextTab[0]);  Write("\n");
	Write(helpBuffer);  Write("\n");
	Bank2Function();  Write("\n");

  Write("READY.\n");

	return 0;
}
